/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2011 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * FormatParser_common_common.cpp
 *
 *  Created on: Mar 1, 2010
 *      Author: metzler
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include <iostream>

#include "CodePatterns/Observer/Notification.hpp"
#include "World.hpp"
#include "ChangeTracker.hpp"
#include "FormatParser_common.hpp"

using namespace std;

/**
 * Constructor.
 */
FormatParser_common::FormatParser_common(FormatParser_Parameters *_parameters) :
  Observer("FormatParser_common"),
  saveStream(NULL)
{
  parameters = _parameters;
  ChangeTracker::getInstance().signOn(this);
  World::getInstance().signOn(this, World::AtomInserted);
  World::getInstance().signOn(this, World::AtomRemoved);
}

/**
 * Destructor.
 */
FormatParser_common::~FormatParser_common() 
{
  ChangeTracker::getInstance().signOff(this);
  World::getInstance().signOff(this, World::AtomInserted);
  World::getInstance().signOff(this, World::AtomRemoved);
  if (parameters != NULL)
    delete parameters;
}

/**
 * Update operation which can be invoked by the observable (which should be the
 * change tracker here).
 */
void FormatParser_common::update(Observable *publisher) {
  if (saveStream != NULL) { // only store when a saveStream is given
    std::vector<atom *> atoms = World::getInstance().getAllAtoms();
    save(saveStream, atoms);
  }
}

/**
 * With this, each format parser is informed about specific changes in the World.
 */
void FormatParser_common::recieveNotification(Observable *publisher, Notification_ptr notification) {
  switch (notification->getChannelNo()) {
    case World::AtomInserted:
      AtomInserted(World::getInstance().lastChanged<atom>()->getId());
      break;
    case World::AtomRemoved:
      AtomRemoved(World::getInstance().lastChanged<atom>()->getId());
      break;
    default:
      ASSERT(0,
          "FormatParser_common::recieveNotification() - unknown notification "
          +toString(notification->getChannelNo())+" received.");
      break;
  }
}

/**
 * The observable can tell when it dies.
 */
void FormatParser_common::subjectKilled(Observable *publisher) {}

/**
 * Sets the output stream for save, so the save() method can be invoked on update
 * automatically.
 *
 * \param ostream where to save the World's state
 */
void FormatParser_common::setOstream(ostream* output) {
  saveStream = output;
}
